<?php

namespace Drupal\term_merge_manager\EventSubscriber;

use Drupal\redirect\Entity\Redirect;
use Drupal\term_merge\TermsMergedEvent;
use Drupal\term_merge_manager\Entity\TermMergeFrom;
use Drupal\term_merge_manager\Entity\TermMergeInto;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Class DefaultSubscriber.
 */
class DefaultSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events['term_merge.terms_merged'] = ['term_merge_merge_action'];

    return $events;
  }

  /**
   * This method is called whenever the term_merge.merge_action event is
   * dispatched.
   *
   * @param \Drupal\term_merge\TermsMergedEvent $event
   */
  public function term_merge_merge_action(TermsMergedEvent $event) {

    $into = $event->getTargetTerm();
    $from = $event->getSourceTerms();

    // Load existing rule or create it.
    $mergeintoid = TermMergeInto::loadIdByTid($into->id());

    if ($mergeintoid === FALSE) {
      $terminto = TermMergeInto::create([
        'tid' => $into->id(),
        'vid' => $into->bundle(),
      ]);
      $terminto->save();
      $mergeintoid = $terminto->id();
    }

    $moduleHandler = \Drupal::service('module_handler');
    $redirectEnabled = FALSE;
    if ($moduleHandler->moduleExists('redirect')) {
      $redirectEnabled = TRUE;
    }

    // Create or update items.
    /** @var \Drupal\taxonomy\TermInterface $item */
    foreach ($from as $index => $item) {
      $vid = $item->bundle();
      $name = $item->getName();

      $from = TermMergeFrom::loadByVidName($vid, $name);
      if ($from === FALSE) {
        $from = TermMergeFrom::create();
      }

      $from->set('tmiid', $mergeintoid);
      $from->set('vid', $vid);
      $from->set('name', $name);
      $from->save();

      // Add redirect if module enabled and auto redirect is checked
      if ($redirectEnabled) {

        $config = \Drupal::config('redirect.settings');
        if (!$config->get('auto_redirect')) {
          return;
        }

        $lang = $item->language()->getId();
        $alias = \Drupal::service('path_alias.manager')->getAliasByPath('/taxonomy/term/'.$item->id());
        $aliasinto = \Drupal::service('path_alias.manager')->getAliasByPath('/taxonomy/term/'.$into->id());

        // Delete all redirects having the same source as this alias.
        redirect_delete_by_path($aliasinto, $lang, FALSE);

        // Create redirect from the old path alias to the new one.
        if ($alias != $aliasinto) {
          if (!redirect_repository()->findMatchingRedirect($alias, [], $lang)) {
            $redirect = Redirect::create();
            $redirect->setSource($alias);
            $redirect->setRedirect('/taxonomy/term/'.$into->id());
            $redirect->setLanguage($lang);
            $redirect->setStatusCode($config->get('default_status_code'));
            $redirect->save();
          }
        }
      }
    }
  }
}
